package com.edgec.browserbackend.browser.service.Impl;

import com.edgec.browserbackend.account.domain.Account;
import com.edgec.browserbackend.account.exception.AccountErrorCode;
import com.edgec.browserbackend.account.repository.AccountRepository;
import com.edgec.browserbackend.browser.ErrorCode.BrowserErrorCode;
import com.edgec.browserbackend.browser.domain.Group;
import com.edgec.browserbackend.browser.domain.IpResource;
import com.edgec.browserbackend.browser.domain.Shop;
import com.edgec.browserbackend.browser.domain.UserShop;
import com.edgec.browserbackend.browser.dto.FilterDto;
import com.edgec.browserbackend.browser.dto.ShopDto;
import com.edgec.browserbackend.browser.repository.GroupRepository;
import com.edgec.browserbackend.browser.repository.IpResourceRepository;
import com.edgec.browserbackend.browser.repository.ShopRepository;
import com.edgec.browserbackend.browser.repository.UserShopRepository;
import com.edgec.browserbackend.browser.service.ShopService;
import com.edgec.browserbackend.common.commons.error.ClientRequestException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.jws.soap.SOAPBinding;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional
public class ShopServiceImpl implements ShopService {

    private final Logger logger = LoggerFactory.getLogger(ShopServiceImpl.class);

    @Autowired
    ShopRepository shopRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    UserShopRepository userShopRepository;

    @Autowired
    GroupRepository groupRepository;

    @Autowired
    IpResourceRepository ipResourceRepository;

    @Override
    public String addShop(String username, Shop shop) {
        Account account = accountRepository.findByName(username);
        String id = null;
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        if (account.getPermission() < 4) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        userShopRepository.findByUsernameAndShopId(username, shop.getId());
        if (shop.getFirstGroupId() != null) {
            transferShop(username, shop.getId(), shop.getFirstGroupId());
        }
        try {
            shop.setOwner(username);
            id = shopRepository.save(shop).getId();
        }catch (Exception e) {
            throw new ClientRequestException(BrowserErrorCode.INFORMATIONNOTCOMPELETE);
        }
        return id;
    }

    @Override
    public List<String> addShops(String username, List<Shop> shops) {
        return null;
    }

    @Override
    public String updateShop(String username, Shop shop) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shop.getId());
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        if (shop.getFirstGroupId() != null) {
            transferShop(username, shop.getId(), shop.getFirstGroupId());
        }

        Shop shop_old = shopRepository.findById(shop.getId()).orElseGet(null);
        if (shop_old == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        try {
            shop_old = shop_old.of(shop);
            shopRepository.save(shop_old);
        }catch (Exception e) {
            logger.error("fail to update", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.INFORMATIONNOTCOMPELETE);
        }
        return shop_old.getId();
    }

    @Override
    public void deleteShop(String username, String shopId) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        if (shop.getIpId() != null && shop.getIp() != null) {
            unBindShop(username, shopId);
        }
        boolean result = userShopRepository.deleteByUsernameAndShopId(username, shopId);
        if (result) {
            shopRepository.deleteById(shopId);
        } else throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
    }

    @Override
    public void bindShop(String username, String shopId, String ipAddr) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        if (shop.getIpId() != null && shop.getIp() != null) {
            unBindShop(username, shopId);
        }
        IpResource ipResource = ipResourceRepository.findByIpAddrAndDeleted(ipAddr, false);
        if (ipResource == null)
            throw new ClientRequestException(BrowserErrorCode.IPNOTEXIST);
        try {
            shop.setIp(ipAddr);
            shop.setIpId(ipResource.getId());
            shop.setIpRegion(ipResource.getIpRegion());
            shopRepository.save(shop);
            List<String> history = ipResource.getBindHistory();
            history.add(shop.getId());
            ipResourceRepository.save(ipResource);
        } catch (Exception e) {
            logger.error("fail to bind shop and ip", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.UNKNOWN);
        }
    }

    @Override
    public void unBindShop(String username, String shopId) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        IpResource ipResource = ipResourceRepository.findByIpAddrAndDeleted(shop.getIp(), false);
        if (ipResource == null)
            throw new ClientRequestException(BrowserErrorCode.IPNOTEXIST);
        try {
            shop.setIp(null);
            shop.setIpId(null);
            shop.setIpRegion(null);
            shopRepository.save(shop);
        } catch (Exception e) {
            logger.error("fail to unbind", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.UNKNOWN);
        }
    }

    @Override
    public void transferShop(String username, String shopId, String groupId) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        Group group = groupRepository.findById(groupId).orElse(null);
        if (account.getPermission() < 4 || userShop == null || !group.getOwner().equals(username)) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        try {
            userShop.setGroupId(groupId);
            userShopRepository.save(userShop);
        } catch (Exception e) {
            logger.error("fail to unbind", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.UNKNOWN);
        }

    }

    @Override
    public void assignShop(String username, String shopId, List<String> users) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        if (account.getPermission() < 8 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        List<Account> accounts = accountRepository.findByNameIn(users);
        if (accounts != null || accounts.size() != users.size())
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        accounts.stream().forEach(x -> {
            try {
                UserShop userShop1 = userShopRepository.findByUsernameAndShopId(x.getName(), shopId);
                if (userShop1 != null)
                    return;
                userShopRepository.save(new UserShop(x.getName(), shopId));
            } catch (Exception e) {
                logger.error("fail to assign", e.getMessage());
            }
        });
    }

    @Override
    public List<ShopDto> getShopList(String username, String groupId, int page, int amout, FilterDto filterDto) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        Group group = groupRepository.findById(groupId).orElse(null);
        if (group == null)
            throw new ClientRequestException(BrowserErrorCode.GROUPNOTEXIST);
        if (account.getPermission() < 8 || !group.getOwner().equals(username)) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        List<String> shopIds = userShopRepository.findByUsernameAndGroupId(username, groupId).stream().
                map(x -> x.getShopId()).collect(Collectors.toList());
        List<Shop> shops;
        if (filterDto.getIpRegion() != null)
            shops = shopRepository.findByIdInAndIpRegionLike(shopIds, filterDto.getIpRegion());
        else if (filterDto.getShopAccount() != null)
            shops = shopRepository.findByIdInAndShopAccountLike(shopIds, filterDto.getShopAccount());
        else if (filterDto.getShopAccount() != null)
            shops = shopRepository.findByIdInAndShopNameLike(shopIds, filterDto.getShopName());
        else
            shops = shopRepository.findByIdIn(shopIds);
        if (shops == null || shops.size() < 1)
            return new ArrayList<>();
        List<ShopDto> shopDtos = new ArrayList<>();
        shops.stream().forEach(x -> {
            IpResource ipResource = ipResourceRepository.findByIpAddrAndDeleted(x.getIp(), false);
            shopDtos.add(ShopDto.of(x, ipResource));
        });
        return shopDtos;
    }
}
